@@ -31,7 +31,7 @@ HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTIO
 CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  
-$Id: data.c 2103 2012-11-28 15:56:10Z beerli $
+$Id: data.c 2116 2012-12-31 03:46:14Z beerli $
 
 -----------------------------------
 add this to the microsatellite data, this will allow to use
@@ -111,7 +111,7 @@ void print_seqdata (world_fmt * world, option_fmt * options, data_fmt * data);
 
 void print_header (FILE * outfile, long pop, world_fmt * world,
                    option_fmt * options, data_fmt * data);
-void create_alleles (data_fmt * data, option_fmt *options);
+MYREAL create_alleles (data_fmt * data, option_fmt *options);
 void addAllele (data_fmt * data, char s[], long locus, long *z);
 void set_numind (data_fmt * data);
 void print_seq_pop (long locus, long pop, world_fmt * world,
@@ -165,6 +165,7 @@ destroy_data (data_fmt * data)
   myfree(data->subloci);
   myfree(data->maxalleles);
   myfree(data->skiploci);
+  myfree(data->locusweight);
 
   // free data from init_data_structure2
   for(pop=0; pop < numpop ; pop++)
@@ -298,8 +299,11 @@ void
 get_new_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *world)
 {
   //long locus;
+  MYREAL mean;
   long pop;
+  long locus;
   long genomes=1;
+  boolean saved_option_murate=FALSE;
   data->hasghost = FALSE;
   // read how many populations and loci and delimiter and title
   // if we have the new haplotype input then there should be no delimiter
@@ -342,26 +346,30 @@ get_new_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *w
     if (options->progress)
         fprintf (stdout, "\n\n");
     init_data_structure3 (data,options);
-
+    if ((options->onlyvariable || options->has_variableandone) && !options->murates_fromdata)
+      {
+	saved_option_murate = TRUE;
+	options->murates_fromdata = TRUE;
+      }
     switch (options->datatype)
     {
     case 'a':
-        create_alleles (data, options);
+        mean = create_alleles (data, options);
         break;
     case 'b':
-        create_alleles (data, options);
+        mean = create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = XBROWN_SIZE;
         break;
     case 'm':
-        create_alleles (data, options);
+        mean = create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = options->micro_stepnum;
         break;
     case 'h':
     case 'n':
     case 'u':
-      create_alleles (data, options);
+      mean = create_alleles (data, options);
       break;
     default: /*DNA types*/
       for (pop = 0; pop < data->loci; pop++)
@@ -374,8 +382,21 @@ get_new_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *w
       {
 	if (strchr (SEQUENCETYPES, options->datatype))
 	  {
-	    find_rates_fromdata(data, options);
+	    find_rates_fromdata(data, options, world);
 	  }
+	else
+	  {
+	    find_rates_fromdata_alleles(data, options, world, mean);
+	  }
+      }
+    if(saved_option_murate)
+      {
+	for (locus=0; locus < data->loci; locus++)
+	  {
+	    options->mu_rates[locus] = 1.0;
+	  }
+	options->murates_fromdata=FALSE;
+	saved_option_murate = FALSE;
       }
 }
 
@@ -383,8 +404,10 @@ void
 get_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *world)
 {
   long locus;
-    long pop;
-    long genomes=1;
+  long pop;
+  long genomes=1;
+  MYREAL mean;
+  boolean saved_option_murate=FALSE;
     data->hasghost = FALSE;
     read_header (infile, data, options);
     genomes =  number_genomes (options->datatype);
@@ -438,26 +461,30 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *world
     if (options->progress)
         fprintf (stdout, "\n\n");
     init_data_structure3 (data,options);
-
+    if ((options->onlyvariable || options->has_variableandone) && !options->murates_fromdata)
+      {
+	saved_option_murate = TRUE;
+	options->murates_fromdata = TRUE;
+      }
     switch (options->datatype)
     {
     case 'a':
-        create_alleles (data, options);
+        mean=create_alleles (data, options);
         break;
     case 'b':
-        create_alleles (data, options);
+        mean=create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = XBROWN_SIZE;
         break;
     case 'm':
-        create_alleles (data, options);
+        mean=create_alleles (data, options);
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = options->micro_stepnum;
         break;
     case 'h':
     case 'n':
     case 'u':
-      create_alleles (data, options);
+      mean=create_alleles (data, options);
       break;
     default: /*DNA types*/
       for (pop = 0; pop < data->loci; pop++)
@@ -470,8 +497,21 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *world
       {
 	if (strchr (SEQUENCETYPES, options->datatype))
 	  {
-	    find_rates_fromdata(data, options);
+	    find_rates_fromdata(data, options, world);
 	  }
+	else
+	  {
+	    find_rates_fromdata_alleles(data, options, world, mean);
+	  }
+      }
+    if(saved_option_murate)
+      {
+	for (locus=0; locus < data->loci; locus++)
+	  {
+	    options->mu_rates[locus] = 1.0;
+	  }
+	options->murates_fromdata=FALSE;
+	saved_option_murate = FALSE;
       }
 }
 
@@ -545,7 +585,9 @@ init_data_structure2 (data_fmt ** data, option_fmt * options, long pop)
 
     (*data)->yy[pop] = (char ****) mymalloc (sizeof (char ***) * indalloc);
     (*data)->indnames[pop] = (char ***) mycalloc (1, sizeof (char **) * indalloc);
+#ifdef DEBUG 
     printf("init_data_structure2: indalloc=%li\n",indalloc);
+#endif
     for (ind = 0; ind < indalloc; ind++)
     {
         (*data)->indnames[pop][ind] =
@@ -635,6 +677,12 @@ init_data_structure3 (data_fmt * data, option_fmt *options)
     data->maxalleles = (long *) mycalloc (1, sizeof (long) * data->loci);
     data->skiploci =
         (boolean *) mycalloc (1, sizeof (boolean) * (data->loci + 1));
+    data->locusweight =
+        (MYREAL *) mycalloc (1, sizeof (MYREAL) * (data->loci + 1));
+    for (locus=0;locus < data->loci; locus++)
+      {
+	data->locusweight[locus]=1.0;
+      }
 }
 
 ///
@@ -2622,7 +2670,7 @@ void find_allele_repeatlength(data_fmt *data, option_fmt *options, long locus)
 }
 
 
-void
+MYREAL
 create_alleles (data_fmt * data, option_fmt *options)
 {
   long n=0;
@@ -2665,7 +2713,7 @@ create_alleles (data_fmt * data, option_fmt *options)
         }
 	if(z==0)
 	  {
-	    data->skiploci[locus] = TRUE;
+	    data->skiploci[locus] = TRUE;// this skips loci with _no_ data
 	    continue;
 	  }
 
@@ -2693,6 +2741,7 @@ create_alleles (data_fmt * data, option_fmt *options)
 	      options->mu_rates[locus] /= mean;
 	  }
       }
+    return mean;
 }
 
 void
