@@ -68,13 +68,14 @@ gamma_loci_like(helper_fmt *helper, MYREAL *oparam, MYREAL *olparam, MYREAL deno
     long locus;
     MYREAL summ = 0.0;
     nr_fmt *nr = helper->nr;
+    MYREAL *locusweight = helper->nr->world->data->locusweight; //invariant loci treatment
     for (locus = 0; locus < nr->world->loci; locus++)
     {
         if (nr->skiploci[locus])
             continue;
         nr->locilikes[locus] =
             gamma_locus_like (nr, oparam, olparam, denom, locus);
-        summ += nr->locilikes[locus];
+        summ += locusweight[locus] * nr->locilikes[locus];
     }
     return summ;
 }
@@ -219,6 +220,7 @@ MYREAL find_gamma_summ(MYREAL lla, MYREAL theta1, nr_fmt *nr, MYREAL *oparam, MY
     MYREAL summ=0.0;
     MYREAL so = oparam[nr->numpop2];
     MYREAL slo = olparam[nr->numpop2];
+    MYREAL *locusweight = helper->nr->world->data->locusweight; //invariant loci treatment
     oparam[nr->numpop2] = la;
     olparam[nr->numpop2] = lla;
     if (olparam[nr->numpop2] > 9.903487553)
@@ -233,7 +235,7 @@ MYREAL find_gamma_summ(MYREAL lla, MYREAL theta1, nr_fmt *nr, MYREAL *oparam, MY
         if (nr->skiploci[locus])
             continue;
         nr->locilikes[locus] = gamma_locus_like (nr, oparam, olparam, denom, locus);
-        summ += nr->locilikes[locus];
+        summ += locusweight[locus] * nr->locilikes[locus]; //invariant loci treatment
     }
     oparam[nr->numpop2] = so;
     olparam[nr->numpop2] = slo;
